<?php

namespace Schaffen\SerasaRemessa\Cnab600;

use Schaffen\SerasaRemessa\Utils\Record;
use Schaffen\SerasaRemessa\Utils\RecordColumn;
use Schaffen\SerasaRemessa\Utils\Util;

class Detail extends Record {
    private $code = 1;
    private $sequence;
    private $operation;
    private $cnpj_branch_digit;
    private $occurrence_date;
    private $contract_end_date;
    private $nature_code;
    private $embratel_code;
    private $person_type;
    private $person_fisrt_document_type;
    private $person_first_document;
    private $discharge_reason;
    private $person_second_document_type;
    private $person_second_document;
    private $person_second_document_uf;

    private $guarantor_type;
    private $guarantor_fisrt_document_type;
    private $guarantor_first_document;
    private $guarantor_second_document_type;
    private $guarantor_second_document;
    private $guarantor_second_document_uf;
    
    // 
    private $person_name;
    private $person_birthdate;
    private $person_father_name;
    private $person_mother_name;
    private $person_full_address;
    private $person_neighborhood;
    private $person_city;
    private $person_uf;
    private $person_cep;
    
    //
    private $unpaid_value;
    private $contract_number;
    private $our_number;
    private $person_phone_ddd;
    private $person_phone_number;
    private $contract_start_date;
    private $contract_value;
    private $communication_type;


    public function __construct($cnpj_branch_digit, array $values, $sequence = null) {
        $this->validate($values);
        $this->setValues($values);
        $this->setSequence($sequence);
        $this->setCnpjBranchDigit($cnpj_branch_digit);
        $this->setColumns();
    }

    public function setSequence($sequence) {
        $this->sequence = $sequence;
    }

    public function setCnpjBranchDigit($cnpj_branch_digit) {
        $this->cnpj_branch_digit = $cnpj_branch_digit;
    }

    private function setValues($values) {
        $this->operation = $values['operation'];
        $this->occurrence_date = Util::dateToInt($values['occurrence_date']);
        $this->contract_end_date = Util::dateToInt($values['contract_end_date']);
        $this->nature_code = $values['nature_code'];
        $this->embratel_code = $values['embratel_code'];
        $this->person_type = $values['person_type'];
        $this->person_fisrt_document_type = $values['person_fisrt_document_type'];
        $this->person_first_document = $values['person_first_document'];
        $this->discharge_reason = $values['discharge_reason'];
        $this->person_second_document_type = $values['person_second_document_type'];
        $this->person_second_document = $values['person_second_document'];
        $this->person_second_document_uf = $values['person_second_document_uf'];
        
        // coobrigado
        $this->guarantor_type = $values['guarantor_type'];
        $this->guarantor_fisrt_document_type = $values['guarantor_fisrt_document_type'];
        $this->guarantor_first_document = $values['guarantor_first_document'];
        $this->guarantor_second_document_type = $values['guarantor_second_document_type'];
        $this->guarantor_second_document = $values['guarantor_second_document'];
        $this->guarantor_second_document_uf = $values['guarantor_second_document_uf'];
        
        // 
        $this->person_name = $values['person_name'];
        $this->person_birthdate = Util::dateToInt($values['person_birthdate']);
        $this->person_father_name = $values['person_father_name'];
        $this->person_mother_name = $values['person_mother_name'];
        $this->person_full_address = Util::onlyLettersNumbersAndSpaces($values['person_full_address']);
        $this->person_neighborhood = $values['person_neighborhood'];
        $this->person_city = $values['person_city'];
        $this->person_uf = $values['person_uf'];
        $this->person_cep = $values['person_cep'];
        
        //
        $this->unpaid_value = $values['unpaid_value'];
        $this->contract_number = Util::onlyNumbers($values['contract_number']);
        $this->our_number = $values['our_number'];
        $this->person_phone_ddd = Util::onlyNumbers($values['person_phone_ddd']);
        $this->person_phone_number = Util::onlyNumbers($values['person_phone_number']);
        $this->contract_start_date = Util::dateToInt($values['contract_start_date']);
        $this->contract_value = $values['contract_value'];
        $this->communication_type = $values['communication_type'];
    }

    private function setColumns() {
        $this->setColumn('code', new RecordColumn($this->code, RecordColumn::TYPE_NUMERIC, 1, 1));
        $this->setColumn('operation', new RecordColumn($this->operation, RecordColumn::TYPE_ALPHABETICAL, 2, 1));
        $this->setColumn('cnpj_branch_digit', new RecordColumn($this->cnpj_branch_digit, RecordColumn::TYPE_ALPHABETICAL, 3, 6));
        $this->setColumn('occurrence_date', new RecordColumn($this->occurrence_date, RecordColumn::TYPE_NUMERIC, 9, 8));
        $this->setColumn('contract_end_date', new RecordColumn($this->contract_end_date, RecordColumn::TYPE_NUMERIC, 17, 8));
        $this->setColumn('nature_code', new RecordColumn($this->nature_code, RecordColumn::TYPE_ALPHANUMERIC, 25, 3));
        $this->setColumn('embratel_code', new RecordColumn($this->embratel_code, RecordColumn::TYPE_ALPHABETICAL, 28, 4));
        $this->setColumn('person_type', new RecordColumn($this->person_type, RecordColumn::TYPE_ALPHABETICAL, 32, 1));
        $this->setColumn('person_fisrt_document_type', new RecordColumn($this->person_fisrt_document_type, RecordColumn::TYPE_ALPHANUMERIC, 33, 1));
        $this->setColumn('person_first_document', new RecordColumn($this->person_first_document, RecordColumn::TYPE_NUMERIC, 34, 15));
        $this->setColumn('discharge_reason', new RecordColumn($this->discharge_reason, RecordColumn::TYPE_ALPHANUMERIC, 49, 2));
        $this->setColumn('person_second_document_type', new RecordColumn($this->person_second_document_type, RecordColumn::TYPE_ALPHANUMERIC, 51, 1));
        $this->setColumn('person_second_document', new RecordColumn($this->person_second_document, RecordColumn::TYPE_ALPHANUMERIC, 52, 15));
        $this->setColumn('person_second_document_uf', new RecordColumn($this->person_second_document_uf, RecordColumn::TYPE_ALPHANUMERIC, 67, 2));
        
        // coobrigado
        $this->setColumn('guarantor_type', new RecordColumn($this->guarantor_type, RecordColumn::TYPE_ALPHABETICAL, 69, 1));
        $this->setColumn('guarantor_fisrt_document_type', new RecordColumn($this->guarantor_fisrt_document_type, RecordColumn::TYPE_ALPHANUMERIC, 70, 1));

        $this->setColumn('guarantor_first_document', new RecordColumn($this->guarantor_first_document, 
            (!empty($this->guarantor_first_document) ? RecordColumn::TYPE_NUMERIC : RecordColumn::TYPE_ALPHANUMERIC), 71, 15));
        
        $this->setColumn('blank_1', new RecordColumn('', RecordColumn::TYPE_ALPHANUMERIC, 86, 2));
        $this->setColumn('guarantor_second_document_type', new RecordColumn($this->guarantor_second_document_type, RecordColumn::TYPE_ALPHANUMERIC, 88, 1));
        $this->setColumn('guarantor_second_document', new RecordColumn($this->guarantor_second_document, RecordColumn::TYPE_ALPHANUMERIC, 89, 15));
        $this->setColumn('guarantor_second_document_uf', new RecordColumn($this->guarantor_second_document_uf, RecordColumn::TYPE_ALPHANUMERIC, 104, 2));
        
        // arrumar posição e ordem
        $this->setColumn('person_name', new RecordColumn($this->person_name, RecordColumn::TYPE_ALPHANUMERIC, 106, 70));
        $this->setColumn('person_birthdate', new RecordColumn($this->person_birthdate, RecordColumn::TYPE_NUMERIC, 176, 8));
        $this->setColumn('person_father_name', new RecordColumn($this->person_father_name, RecordColumn::TYPE_ALPHANUMERIC, 184, 70));
        $this->setColumn('person_mother_name', new RecordColumn($this->person_mother_name, RecordColumn::TYPE_ALPHANUMERIC, 254, 70));
        $this->setColumn('person_full_address', new RecordColumn($this->person_full_address, RecordColumn::TYPE_ALPHANUMERIC, 324, 45));
        $this->setColumn('person_neighborhood', new RecordColumn($this->person_neighborhood, RecordColumn::TYPE_ALPHANUMERIC, 369, 20));
        $this->setColumn('person_city', new RecordColumn($this->person_city, RecordColumn::TYPE_ALPHANUMERIC, 389, 25));
        $this->setColumn('person_uf', new RecordColumn($this->person_uf, RecordColumn::TYPE_ALPHABETICAL, 414, 2));
        $this->setColumn('person_cep', new RecordColumn($this->person_cep, RecordColumn::TYPE_NUMERIC, 416, 8));

        $this->setColumn('unpaid_value', new RecordColumn($this->unpaid_value, RecordColumn::TYPE_NUMERIC, 424, 15));
        $this->setColumn('contract_number', new RecordColumn($this->contract_number, RecordColumn::TYPE_NUMERIC, 439, 16));
        $this->setColumn('our_number', new RecordColumn($this->our_number, RecordColumn::TYPE_ALPHANUMERIC, 455, 9));

        $this->setColumn('person_address_rest', new RecordColumn($this->getColumn('person_full_address')->getRest(), RecordColumn::TYPE_ALPHANUMERIC, 464, 25));

        $this->setColumn('person_phone_ddd', new RecordColumn($this->person_phone_ddd, RecordColumn::TYPE_NUMERIC, 489, 4));
        $this->setColumn('person_phone_number', new RecordColumn($this->person_phone_number, RecordColumn::TYPE_NUMERIC, 493, 9));
        $this->setColumn('contract_start_date', new RecordColumn($this->contract_start_date, RecordColumn::TYPE_NUMERIC, 502, 8));
        $this->setColumn('contract_value', new RecordColumn($this->contract_value, RecordColumn::TYPE_NUMERIC, 510, 15));
        $this->setColumn('complementary_data', new RecordColumn('', RecordColumn::TYPE_ALPHANUMERIC, 525, 1));
        $this->setColumn('blank_2', new RecordColumn('', RecordColumn::TYPE_ALPHANUMERIC, 526, 5));
        $this->setColumn('communication_type', new RecordColumn($this->communication_type, RecordColumn::TYPE_ALPHANUMERIC, 531, 1));
        $this->setColumn('blank_3', new RecordColumn('', RecordColumn::TYPE_ALPHANUMERIC, 532, 2));
        $this->setColumn('errors_codes', new RecordColumn('', RecordColumn::TYPE_ALPHANUMERIC, 534, 60));

        $this->setColumn('sequence', new RecordColumn($this->sequence, RecordColumn::TYPE_NUMERIC, 594, 7));
    }

    public function validate($values) {
        $this->validateOperation($values['operation']);
        // Fazer mais validações
    }

    public function validateOperation($operation) {
        if($operation != 'I' && $operation != 'E') {
            throw new \Exception('Informed operation does not exists');
        }
    }
}